<?php

namespace IPS\advancedfooter\Footer;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Swiper Slider Node
 */
class _Block extends \IPS\Node\Model implements \IPS\Node\Permissions
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'advancedfooter_blocks';

	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'block_';

	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';

	/**
	 * @brief	[Node] Parent Node ID Database Column
	 */
	public static $parentNodeColumnId = 'column_id';

	/**
	 * @brief	[Node] Parent Node Class
	 */
	public static $parentNodeClass = 'IPS\advancedfooter\Footer\Column';
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'position';

	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'menu__advancedfooter_advancedfooter_blocks';

	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = TRUE;

	/**
	 * @brief	[Node] ACP Restrictions
	 */
	protected static $restrictions = array(
		'app'		=> 'advancedfooter',
		'module'	=> 'blocks',
		'prefix' 	=> 'block_',
		'all'		=> 'blcoks_manage',
	);

		/**
		 * @brief	[Node] App for permission index
		 */
		public static $permApp = 'advancedfooter';

		/**
		 * @brief	[Node] Type for permission index
		 */
		public static $permType = 'footerblocks';

		/**
		 * @brief	The map of permission columns
		 */
		public static $permissionMap = array(
			'view' 		=> 'view',
		);

		/**
		 * @brief	[Node] Moderator Permission
		 */
		public static $modPerm = 'footerblocks';

		/**
		 * @brief	[Node] Prefix string that is automatically prepended to permission matrix language strings
		 */
		public static $permissionLangPrefix = 'block_';

		/**
		 * @brief	Bitwise values for members_bitoptions field
		 */
		public static $bitOptions = array(
			'bitoptions' => array(
				'bitoptions' => array(
					'quicklinksblank'			=> 1,
					'usecustomwidth'			=> 2,
					'customblockcolors'	=> 4,
				)
			)
		);

				/**
				 * [Node] Get whether or not this node is enabled
				 *
				 * @note	Return value NULL indicates the node cannot be enabled/disabled
				 * @return	bool|null
				 */
				protected function get__enabled()
				{
					return $this->enabled;
				}

				/**
				 * [Node] Set whether or not this node is enabled
				 *
				 * @param	bool|int	$enabled	Whether to set it enabled or disabled
				 * @return	void
				 */
				protected function set__enabled( $enabled )
				{
					$this->enabled	= $enabled;
				}
				/**
				 * [Node] Get Node Title
				 *
				 * @return	string
				 */
				protected function get__title()
				{

					try {

					if($this->title) {
						return $this->title;
					} else if($this->blocks == 'custom')
						{
							if($this->blockscustomtype == 'customtypeeditor')
							{
								return \IPS\Member::loggedIn()->language()->addToStack('block_customb') . ", " . \IPS\Member::loggedIn()->language()->addToStack('block_customb_editor');
							} else {
								return \IPS\Member::loggedIn()->language()->addToStack('block_customb') . ", " . \IPS\Member::loggedIn()->language()->addToStack('block_customb_html');
							}
						}
						else if($this->blocks == 'members')
						{
							return \IPS\Member::loggedIn()->language()->addToStack('block_afmembers');
						}
						else if($this->blocks == 'quicklinks')
						{
							return \IPS\Member::loggedIn()->language()->addToStack('block_quicklinks');
						}
						else if($this->blocks == 'sociallinks')
						{
							return \IPS\Member::loggedIn()->language()->addToStack('block_sociallinks');
						}
						else if($this->blocks == 'forums')
						{
							return \IPS\Member::loggedIn()->language()->addToStack('forums');
						}
						else if($this->blocks == 'gallery')
						{
							return \IPS\Member::loggedIn()->language()->addToStack('gallery_title');
						}
						else if($this->blocks == 'downloads')
						{
							return \IPS\Member::loggedIn()->language()->addToStack('downloads');
						}
						else if($this->blocks == 'pages')
						{
							return \IPS\Member::loggedIn()->language()->addToStack('menutab__cms');
						}

					}
					catch( \OutOfRangeException $e )
					{
						return '';
					}

				}


				/**
				 * [Node] Get Node Description
				 *
				 * @return	string
				 */
				protected function get__description()
				{

				try {
					if($this->blocks == 'pages')
					{
						if($this->pages_cat == "*"){
							return \IPS\Member::loggedIn()->language()->addToStack( 'ss_blockfeed_pages_all', NULL, array( 'sprintf' => array( $this->contentlimit, \IPS\cms\Databases::load( $this->pages_db )->_title ) ) );
						}
						else
						{
							$cats = array();
							foreach( explode( ',', $this->pages_cat ) AS $id )
							{
								$cats[ $id ] = \IPS\cms\Categories::load($id)->_title;
							}
							return \IPS\Member::loggedIn()->language()->addToStack( 'ss_blockfeed_pages', NULL, array( 'sprintf' => array( $this->contentlimit, \IPS\cms\Databases::load( $this->pages_db )->_title, implode( ', ', $cats ) ) ) );
						}
					}
					else if($this->blocks == 'downloads')
					{
						if($this->downloads == "*"){
							return \IPS\Member::loggedIn()->language()->addToStack( 'ss_blockfeed_downloads_all', NULL, array( 'sprintf' => array( $this->contentlimit ) ) );
						}
						else
						{
							$cats = array();
							foreach( explode( ',', $this->downloads ) AS $id )
							{
								$cats[ $id ] = \IPS\downloads\Category::load($id)->_title;
							}
							return \IPS\Member::loggedIn()->language()->addToStack( 'ss_blockfeed_downloads', NULL, array( 'sprintf' => array( $this->contentlimit, implode( ', ', $cats ) ) ) );
						}
					}
					else if($this->blocks == 'gallery')
					{
						if($this->gallery == "*"){
							return \IPS\Member::loggedIn()->language()->addToStack( 'ss_blockfeed_gallery_all', NULL, array( 'sprintf' => array( $this->contentlimit ) ) );
						}
						else
						{
							$cats = array();
							foreach( explode( ',', $this->gallery ) AS $id )
							{
								$cats[ $id ] = \IPS\gallery\Category::load($id)->_title;
							}
							return \IPS\Member::loggedIn()->language()->addToStack( 'ss_blockfeed_gallery', NULL, array( 'sprintf' => array( $this->contentlimit, implode( ', ', $cats ) ) ) );
						}
					}
					else if($this->blocks == 'forums')
					{
						if($this->forums == "*"){
							return \IPS\Member::loggedIn()->language()->addToStack( 'ss_blockfeed_forums_all', NULL, array( 'sprintf' => array( $this->contentlimit ) ) );
						}
						else
						{
							$cats = array();
							foreach( explode( ',', $this->forums ) AS $id )
							{
								$cats[ $id ] = \IPS\forums\Forum::load($id)->_title;
							}
							return \IPS\Member::loggedIn()->language()->addToStack( 'ss_blockfeed_forums', NULL, array( 'sprintf' => array( $this->contentlimit, implode( ', ', $cats ) ) ) );
						}
					}
				}
				catch( \OutOfRangeException $e )
				{
					return '';
				}
			}
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$form->addTab('block_blockstab');
		$form->addHTML('<style>#block_quicklinks>.ipsFieldRow_content>.ipsField_stack,#block_sociallinks>.ipsFieldRow_content>.ipsField_stack{max-width: 100%;}</style>');
		$toggles = array(
			'custom'	=> array('block_blockscustomtype'),
			'members'	=>array('block_afmembers'),
		 'quicklinks'	=> array('block_quicklinks', 'block_quicklinksblank', 'block_quicklinkscols'),
		 'sociallinks'  => array('block_sociallinks'),
	 	);
		if ( \IPS\Application::appIsEnabled( 'forums' ) || \IPS\Application::appIsEnabled( 'gallery' ) || \IPS\Application::appIsEnabled( 'downloads' ) || \IPS\Application::appIsEnabled( 'cms' ))
		{
			$options	= array();
			$appsfeedOptions	= 'afa_apps_feed';
		}
		else
		{
			$options	= NULL;
			$appsfeedOptions	= NULL;
		}
		if ( \IPS\Application::appIsEnabled( 'forums' ) )
		{
			$options['forums'] = 'forums';
			$toggles['forums'] = array('block_forums', 'block_contentlimit', 'block_status', 'block_sorton', 'block_sortdir');
		}
		if ( \IPS\Application::appIsEnabled( 'gallery' ) )
		{
			$options['gallery'] = 'gallery_title';
			$toggles['gallery'] = array('block_gallery', 'block_contentlimit', 'block_status', 'block_sorton', 'block_sortdir');
		}
		if ( \IPS\Application::appIsEnabled( 'downloads' ) )
		{
			$options['downloads'] = 'downloads';
			$toggles['downloads'] = array('block_downloads', 'block_contentlimit', 'block_status','block_sorton', 'block_sortdir');
		}
		if ( \IPS\Application::appIsEnabled( 'cms' ) )
		{
			$options['pages'] = 'menutab__cms';
			$toggles['pages'] = array('block_pages_db', 'block_contentlimit', 'block_status', 'block_sorton', 'block_sortdir');
		}
		$blocks = array(
			'afa_custom_blocks' => array(
				'custom'	=> 'block_customb',
				'members'	=> 'block_afmembers',
				'quicklinks'  => 'block_quicklinks',
				'sociallinks' => 'block_sociallinks'
			),
			$appsfeedOptions	=> $options
		);
		$form->add( new \IPS\Helpers\Form\Select( 'block_blocks', $this->id ? $this->blocks : "custom", TRUE, array( 'options' => array_filter($blocks), 'toggles' => $toggles ), NULL, NULL, NULL, 'block_blocks'));
		 $form->add( new \IPS\Helpers\Form\Select( 'block_blockscustomtype', $this->id ? $this->blockscustomtype : "customtypeeditor", FALSE, array(
			 'options' => array(
				 'customtypeeditor'  => 'block_customb_editor',
				 'customtypecm' => 'block_customb_html',
			 ),
			 'toggles' => array(
				 'customtypeeditor'	=> array(
					 'block_econtent',
					 'block_econtenttruncate'
				 ),
				'customtypecm'	=> array(
					'block_ccontent',
				)
			) ), NULL, NULL, NULL, 'block_blockscustomtype'));
		$form->add( new \IPS\Helpers\Form\Text( 'block_title', $this->title ? $this->title : "", FALSE, array( 'maxLength' => 255 ), NULL, NULL, NULL, 'block_title' ) );
		$form->add( new \IPS\Helpers\Form\Editor( 'block_econtent', $this->blockscustomtype === 'customtypeeditor' ? $this->content : NULL, FALSE, array( 'app' => 'advancedfooter', 'key' => 'advancedfooter', 'autoSaveKey' => 'block_econtent' ), NULL, NULL, NULL, 'block_econtent' ) );
		$form->add( new \IPS\Helpers\Form\Codemirror( 'block_ccontent', $this->blockscustomtype === 'customtypecm' ? $this->content : NULL, FALSE, array( 'height' => 200 ), NULL, NULL, NULL, 'block_ccontent' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'block_econtenttruncate', $this->id ? $this->econtenttruncate : "4", FALSE, array(), NULL, NULL, NULL, 'block_econtenttruncate' ) );
		$members =  $this->id ? array_map( array( 'IPS\Member', 'load' ), explode( "\n", $this->afmembers ) ) : NULL;
		if ( !empty( $this->afmembers ) )
		{
			foreach( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( '*', 'core_members', array( \IPS\Db::i()->in( 'member_id', explode( "\n", $this->afmembers ) ) ) ), 'IPS\Member' ) AS $member )
			{
				$members[] = $member;
			}
		}
		$form->add( new \IPS\Helpers\Form\Member( 'block_afmembers', $members, FALSE, array( 'multiple' => NULL ), NULL, NULL, NULL, 'block_afmembers' ) );
		$form->add( new \IPS\Helpers\Form\Stack( 'block_quicklinks', $this->id ? json_decode( $this->links, true ) : array(), FALSE, array( 'stackFieldType' => '\IPS\advancedfooter\Links\Links', 'key' => array('placeholder' => 'http://example.com'), 'value' => array('placeholder' => '<i class="fa fa-facebook"> Facebook') ), NULL, NULL, NULL, 'block_quicklinks' ) );
		$form->add( new \IPS\Helpers\Form\Stack( 'block_sociallinks', $this->id ? json_decode( $this->links, true ) : array(), FALSE, array( 'stackFieldType' => '\IPS\advancedfooter\Links\Links', 'key' => array('placeholder' => 'https://www.facebook.com/'), 'value' => array('placeholder' => 'fa fa-facebook') ), NULL, NULL, NULL, 'block_sociallinks' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'block_quicklinkscols', $this->id ? $this->quicklinkscols : "2", FALSE, array('min'	=> 0, 'max'	=> 10), NULL, NULL, NULL, 'block_quicklinkscols' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'block_quicklinksblank', $this->id ? $this->bitoptions['quicklinksblank'] : 0, FALSE, array(), NULL, NULL, NULL, 'block_quicklinksblank' ) );

		/* Forum topics*/
		if ( \IPS\Application::appIsEnabled( 'forums' ) )
		{
			$form->add( new \IPS\Helpers\Form\Node( 'block_forums', ( $this->id and $this->forums != '*' ) ? $this->forums : 0, FALSE, array( 'class' => 'IPS\forums\Forum', 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'block_forums' ) );
		}

		/* Gallery images*/
		if ( \IPS\Application::appIsEnabled( 'gallery' ) )
		{
			$form->add( new \IPS\Helpers\Form\Node( 'block_gallery', ( $this->id and $this->gallery != '*' ) ? $this->gallery : 0, FALSE, array( 'class' => 'IPS\gallery\Category', 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'block_gallery' ) );
		}

		/* Download items*/
		if ( \IPS\Application::appIsEnabled( 'downloads' ) )
		{
			$form->add( new \IPS\Helpers\Form\Node( 'block_downloads', ( $this->id and $this->downloads != '*' ) ? $this->downloads : 0, FALSE, array( 'class' => 'IPS\downloads\Category', 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'block_downloads' ) );
		}

		/* Pages */
		if ( \IPS\Application::appIsEnabled( 'cms' ) )
		{
			$disabled = array();
			$block_pages_db = array();
			foreach( \IPS\cms\Databases::databases() as $db )
			{
				$Cats[ $db->id ] = array( 'block_pages_cat_' . $db->id );
				$dbIDs[] = $db->id;
				\IPS\Member::loggedIn()->language()->words['block_pages_cat_' . $db->id] = "Categories";
				if ( $db->page_id )
				{
					$block_pages_db[ $db->id ] = $db->_title;
				}
				else
				{
					$disabled[] = $db->id;
					$block_pages_db[ $db->id ] = \IPS\Member::loggedIn()->language()->addToStack( 'block_diabled_pages_database', NULL, array( 'sprintf' => array( $db->_title ) ) );
				}
			}

			if ( !empty( $block_pages_db ) )
			{
				$form->add( new \IPS\Helpers\Form\Select( 'block_pages_db', explode( ',', $this->pages_db ), TRUE, array( 'options' => $block_pages_db, 'disabled' => $disabled, 'toggles' => $Cats, 'multiple' => FALSE ), NULL, NULL, NULL, 'block_pages_db' ) );
				foreach( $dbIDs as $dbID )
				{
					$form->add( new \IPS\Helpers\Form\Node( 'block_pages_cat_' . $dbID, ( $this->id and $this->pages_cat != '*' ) ? $this->pages_cat : 0, FALSE, array( 'class' => '\IPS\cms\Categories' . $dbID, 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'block_pages_cat_' . $dbID ) );
				}
			}
		}

		/* Content options */
		if ( \IPS\Application::appIsEnabled( 'forums' ) || \IPS\Application::appIsEnabled( 'gallery' ) || \IPS\Application::appIsEnabled( 'downloads' ) || \IPS\Application::appIsEnabled( 'cms' ))
		{
			$form->add( new \IPS\Helpers\Form\Number( 'block_contentlimit', $this->id ? $this->contentlimit : "6", FALSE, array(), NULL, NULL, NULL, 'block_contentlimit' ) );
			$form->add( new \IPS\Helpers\Form\CheckboxSet( 'block_status', $this->status ? explode( ',', $this->status ) : array(), FALSE, array(
				'options' => array(
				    'pinned'      	=> 'pinned',
				    'unpinned'      => 'unpinned',
				    'featured'    	=> 'featured',
				    'unfeatured'    => 'unfeatured',
				)
			), NULL, NULL, NULL, 'block_status' ) );
			$form->add( new \IPS\Helpers\Form\Select( 'block_sorton', $this->id ? $this->sorton : "id", FALSE, array(
				'options' => array(
						'id'					=> 'sort_on_id',
						'start_date' 			=> 'sort_on_startdate',
						'last_updated' 			=> 'sort_on_recentupdated',
						'title'					=> 'sort_on_title',
						'views'					=> 'sort_on_views',
						'rating'				=> 'sort_on_rating',
					) ), NULL, NULL, NULL, 'block_sorton' ) );
			$form->add( new \IPS\Helpers\Form\Select( 'block_sortdir', $this->id ? $this->sortdir : "DESC", FALSE, array(
				'options' => array(
					'DESC' 	=> 'sort_dir_desc',
					'ASC' 	=> 'sort_dir_asc',
				) ), NULL, NULL, NULL, 'block_sortdir' ) );
			}

				$form->addTab('block_settingsstab');
				$form->add( new \IPS\Helpers\Form\YesNo( 'block_usecustomwidth', $this->id ? $this->bitoptions['usecustomwidth'] : 0, FALSE, array( 'togglesOn' => array( 'block_blockwidth' ) ), NULL, NULL, NULL, 'block_usecustomwidth' ) );
				$form->add( new \IPS\Helpers\Form\Number( 'block_blockwidth', $this->id ? $this->blockwidth : "25", FALSE, array('min'	=> 10, 'max'	=> 100), NULL, NULL, NULL, 'block_blockwidth' ) );
				$form->add( new \IPS\Helpers\Form\YesNo( 'block_customblockcolors', $this->id ? $this->bitoptions['customblockcolors'] : 0, FALSE, array( 'togglesOn' => array( 'block_customblockbg', 'block_customblocktitlecolor', 'block_customblockcontentcolor' ) ), NULL, NULL, NULL, 'block_customblockcolors' ) );
				$form->add( new \IPS\Helpers\Form\Color( 'block_customblockbg', $this->id ? $this->customblockbg : '#2e3033', FALSE, array(), NULL, NULL, NULL, 'block_customblockbg' ) );
				$form->add( new \IPS\Helpers\Form\Color( 'block_customblocktitlecolor', $this->id ? $this->customblocktitlecolor : '#d9d9d9', FALSE, array(), NULL, NULL, NULL, 'block_customblocktitlecolor' ) );
				$form->add( new \IPS\Helpers\Form\Color( 'block_customblockcontentcolor', $this->id ? $this->customblockcontentcolor : '#d9d9d9', FALSE, array(), NULL, NULL, NULL, 'block_customblockcontentcolor' ) );

	}
	/**
	 * [Node] Format form values from add/edit form for save
	 *
	 * @param	array	$values	Values from the form
	 * @return	array
	 */
	public function formatFormValues( $values )
	{
		/* Save bitoptions */
		foreach ( array( 'quicklinksblank', 'usecustomwidth', 'customblockcolors' ) as $k )
		{
			if ( isset( $values[ "block_{$k}" ] ) )
			{
				$this->bitoptions[ $k ] = $values["block_{$k}"];
				unset( $values["block_{$k}"] );
			}
		}
		/* Save custom blocks */
		if( isset( $values['block_blockscustomtype'] ) AND ( isset( $values['block_econtent'] ) OR $values['block_ccontent'] ) )
		{
			$values['content']			= $values['block_blockscustomtype'] === 'customtypeeditor' ? $values['block_econtent'] : $values['block_ccontent'];
			unset( $values['block_econtent'], $values['block_ccontent'] );
			$this->content	= $values['content'];
		}
		/*Save*/
		if ( !$this->id )
		{
			$this->save();
		}
		/* Save forum topics */
		if ( \IPS\Application::appIsEnabled( 'forums' ) && isset( $values['block_forums'] ) )
		{
			if ( $values['block_forums'] == 0 )
			{
				$values['block_forums'] = '*';
			}
			else
			{
				$forums = array();
				foreach ( (array) $values['block_forums'] as $forum )
				{
					$forums[] = $forum->_id;
				}

				$values['block_forums'] = ( implode( ',', $forums ) );
			}
		}
		/* Save gallery */
		if ( \IPS\Application::appIsEnabled( 'gallery' ) && isset( $values['block_gallery'] ) )
		{
			if ( $values['block_gallery'] == 0 )
			{
				$values['block_gallery'] = '*';
			}
			else
			{
				$gcats = array();
				foreach ( $values['block_gallery'] as $gcat )
				{
					$gcats[] = $gcat->_id;
				}

				$values['block_gallery'] = ( implode( ',', $gcats ) );
			}
		}
		/* Save downloads */
		if ( \IPS\Application::appIsEnabled( 'downloads' ) && isset( $values['block_downloads'] ) )
		{
			if ( $values['block_downloads'] == 0 )
			{
				$values['block_downloads'] = '*';
			}
			else
			{
				$dcats = array();
				foreach ( $values['block_downloads'] as $dcat )
				{
					$dcats[] = $dcat->_id;
				}

				$values['block_downloads'] = ( implode( ',', $dcats ) );
			}
		}
		/* Save Pages */
		if ( \IPS\Application::appIsEnabled( 'cms' ) && $values['block_pages_db'] )
		{
			foreach( \IPS\cms\Databases::databases() as $db )
			{
				if ( isset( $values[ 'block_pages_cat_' . $db->id ] ) && ( $db->id == $values['block_pages_db'] ) )
				{
					if ( $values[ 'block_pages_cat_' . $db->id ] == 0 )
					{
						$values['block_pages_cat'] = '*';
					}
					else
					{
						$pagescats = array();
						foreach ( $values[ 'block_pages_cat_' . $db->id ] as $cat )
						{
							$pagescats[] = $cat->_id;
						}

						$values['block_pages_cat'] = ( implode( ',', $pagescats ) );
					}

				}

				unset( $values[ 'block_pages_cat_' . $db->id ] );
			}
		}
		/* Save Links */
		if( ( isset( $values['block_quicklinks'] ) OR $values['block_sociallinks'] ) )
		{
			$values['links']			= $values['block_blocks'] === 'quicklinks' ? json_encode( array_filter( $values['block_quicklinks'], function( $value ) { return (bool) $value['key']; } ) ) : json_encode( array_filter( $values['block_sociallinks'], function( $value ) { return (bool) $value['key']; } ) );

			unset( $values['block_quicklinks'], $values['block_sociallinks'] );
			$this->links	= $values['links'];
		}




		$values['block_afmembers'] = \IPS\Helpers\Form\Member::stringValue($values['block_afmembers']);

		return $values;
		/* return */
		parent::saveForm( $values );
	}


	/***************************
	 *
	 * Get content items
	 *
	 **************************/
	public function items()
	{
		$contents = array();

		switch ( $this->blocks )
		{
			case 'pages':
				$contents = $this->pagesRecords(); break;

			case 'downloads':
				$contents = $this->downloadsItems(); break;

			case 'gallery':
				$contents = $this->galleryImages(); break;

			case 'forums':
			default:
				$contents = $this->forumTopics(); break;
		}

		return $contents;
	}

	/***************************
	 *
	 * Get contents from Forums
	 *
	 **************************/
		protected function forumTopics()
	{

		switch ( $this->sorton )
		{
			case 'id':
				$sort = "tid"; break;

			case 'start_date':
				$sort = "start_date"; break;

			case 'last_updated':
				$sort = "last_real_post"; break;

			case 'title':
				$sort = "title"; break;

			case 'views':
				$sort = "views"; break;

			case 'rating':
				$sort = "topic_rating_total"; break;

			default:
				$sort = "tid"; break;
		}
		$where[] = array( 'forums_forums.password IS NULL AND forums_forums.can_view_others=1' );
		$where[] = array( "approved=1 AND topic_archive_status=0" );
		$limit = $this->contentlimit ? $this->contentlimit : 6;
		if ( $this->forums !== '*' )
		{
			$where[] = array( "forum_id IN (" . $this->forums . ")" );
		}
		if ( !empty( $this->status ) )
		{
			$status = explode(",", $this->status );
			if ( in_array( 'pinned', $status ) )
			{
				$state[] = 'pinned=1';
			}
			if ( in_array( 'unpinned', $status ) )
			{
				$state[] = 'pinned=0';
			}
			if ( in_array( 'featured', $status ) )
			{
				$state[] = 'featured=1';
			}
			if ( in_array( 'unfeatured', $status ) )
			{
				$state[] = 'featured=0';
			}
			$where[] = array('(' . implode(' OR ', $state) . ')');
		}





		if ( \IPS\Application::appIsEnabled( 'forums' ))
		{

			try
			{
				$topics = \IPS\forums\Topic::getItemsWithPermission( $where, $sort . " " . $this->sortdir, $limit );
				return $topics;
			}
			catch ( \OutOfRangeException $ex ){}
		}


	}
	/***************************
	 *
	 * Get contents from Gallery
	 *
	 **************************/
	protected function galleryImages()
	{
		switch ( $this->sorton )
		{
			case 'id':
				$sort = "image_id"; break;

			case 'start_date':
				$sort = "image_date"; break;

			case 'last_updated':
				$sort = "image_updated"; break;

			case 'title':
				$sort = "image_caption"; break;

			case 'views':
				$sort = "image_views"; break;

			case 'rating':
				$sort = "image_ratings_total"; break;

			default:
				$sort = "image_id"; break;
		}

		$where[] = array("image_approved=1");
		$limit = $this->contentlimit ? $this->contentlimit : 6;
		if ( $this->gallery !== '*' )
		{
			$where[] = array("image_category_id IN (". $this->gallery .")");
			//$where[] = array( 'image_category_id=?', $this->gallery );
		}

		if ( !empty( $this->status ) )
		{
			$status = explode(",", $this->status );
			if ( in_array( 'pinned', $status ) )
			{
				$state[] = 'image_pinned=1';
			}
			if ( in_array( 'unpinned', $status ) )
			{
				$state[] = 'image_pinned=0';
			}
			if ( in_array( 'featured', $status ) )
			{
				$state[] = 'image_feature_flag=1';
			}
			if ( in_array( 'unfeatured', $status ) )
			{
				$state[] = 'image_feature_flag=0';
			}
			$where[] = array('(' . implode(' OR ', $state) . ')');
		}
		if ( \IPS\Application::appIsEnabled( 'gallery' ) )
		{
			$galleryimages = \IPS\gallery\Image::getItemsWithPermission( $where, $sort . " " . $this->sortdir, $limit );
			return $galleryimages;
		}

	}
	/***************************
	 *
	 * Get contents from Downloads
	 *
	 **************************/
	protected function downloadsItems()
	{
		switch ( $this->sorton )
		{
			case 'id':
				$sort = "file_id"; break;

			case 'start_date':
				$sort = "file_submitted"; break;

			case 'last_updated':
				$sort = "file_updated"; break;

			case 'title':
				$sort = "file_name"; break;

			case 'views':
				$sort = "file_views"; break;

			case 'rating':
				$sort = "file_rating"; break;

			default:
				$sort = "file_id"; break;
		}

		$where[] = array( "file_open=1" );
		$limit = $this->contentlimit ? $this->contentlimit : 6;
		if ( $this->downloads !== '*' )
		{
			$where[] = array( "file_cat IN (". $this->downloads .")" );
		}

		if ( !empty( $this->status ) )
		{
			$status = explode(",", $this->status );
			if ( in_array( 'pinned', $status ) )
			{
				$state[] = 'file_pinned=1';
			}
			if ( in_array( 'unpinned', $status ) )
			{
				$state[] = 'file_pinned=0';
			}
			if ( in_array( 'featured', $status ) )
			{
				$state[] = 'file_featured=1';
			}
			if ( in_array( 'unfeatured', $status ) )
			{
				$state[] = 'file_featured=0';
			}
			$where[] = array('(' . implode(' OR ', $state) . ')');
		}
		if ( \IPS\Application::appIsEnabled( 'downloads' ) )
		{
			$downloads = \IPS\downloads\File::getItemsWithPermission( $where, $sort . " " . $this->sortdir, $limit );
			return $downloads;
		}
	}
	/***************************
	 *
	 * Get contents from Pages
	 *
	 **************************/
	protected function pagesRecords()
	{
		switch ( $this->sorton )
		{
			case 'id':
				$sort = "primary_id_field"; break;

			case 'start_date':
				$sort = "record_publish_date"; break;

			case 'last_updated':
				$sort = "record_updated"; break;

			case 'title':
				$sort = "record_dynamic_furl"; break;

			case 'views':
				$sort = "record_views"; break;

			case 'rating':
				$sort = "record_rating"; break;

			default:
				$sort = "primary_id_field"; break;
		}

		$recordsClass = 'IPS\cms\Records' . $this->pages_db;
		$where[] = array("record_approved=1");
		$limit = $this->contentlimit ? $this->contentlimit : 6;
		if ( $this->pages_cat !== '*' )
		{
			$where[] = array( "record_approved=1 && category_id IN (". $this->pages_cat .")" );
		}

		if ( !empty( $this->status ) )
		{
			$status = explode(",", $this->status );
			if ( in_array( 'pinned', $status ) )
			{
				$state[] = 'record_pinned=1';
			}
			if ( in_array( 'unpinned', $status ) )
			{
				$state[] = 'record_pinned=0';
			}
			if ( in_array( 'featured', $status ) )
			{
				$state[] = 'record_featured=1';
			}
			if ( in_array( 'unfeatured', $status ) )
			{
				$state[] = 'record_featured=0';
			}
			$where[] = array('(' . implode(' OR ', $state) . ')');
		}
		if ( \IPS\Application::appIsEnabled( 'cms' ))
		{
			$records = $recordsClass::getItemsWithPermission( $where, $sort . " " . $this->sortdir, $limit );
			try
			{
				$database = \IPS\cms\Databases::load( $this->pages_db );
				if ( $database->page_id )
				{
					return $records;
				}
			}
			catch ( \OutOfRangeException $ex ){}
		}
	}
	/**
	 * Search
	 *
	 * @param	string		$column	Column to search
	 * @param	string		$query	Search query
	 * @param	string|null	$order	Column to order by
	 * @param	mixed		$where	Where clause
	 * @return	array
	 */
	public static function search( $column, $query, $order=NULL, $where=array() )
	{
		if ( $column === '_title' )
		{
			$column	= 'block_title';
		}

		if( $order == '_title' )
		{
			$order	= 'block_title';
		}

		return parent::search( $column, $query, $order, $where );
	}

}
