<?php

namespace IPS\advancedfooter\Footer;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Swiper Slider Node
 */
class _Column extends \IPS\Node\Model
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'advancedfooter_columns';

	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'column_';

	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';

	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'position';

	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'menu__advancedfooter_advancedfooter_blocks';

	/**
	 * @brief	[Node] Subnode class
	 */
	public static $subnodeClass = 'IPS\advancedfooter\Footer\Block';

	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = TRUE;

	/**
	 * @brief	[Node] ACP Restrictions
	 */
	protected static $restrictions = array(
		'app'		=> 'advancedfooter',
		'module'	=> 'blocks',
		'prefix' 	=> 'af_'
	);

	/**
	 * @brief	Bitwise values for members_bitoptions field
	 */
	public static $bitOptions = array(
		'bitoptions' => array(
			'bitoptions' => array(
				'usecustombg'			=> 1
			)
		)
	);

	/**
	 * [Node] Get whether or not this node is enabled
	 *
	 * @note	Return value NULL indicates the node cannot be enabled/disabled
	 * @return	bool|null
	 */
	protected function get__enabled()
	{
		return $this->enabled;
	}

	/**
	 * [Node] Set whether or not this node is enabled
	 *
	 * @param	bool|int	$enabled	Whether to set it enabled or disabled
	 * @return	void
	 */
	protected function set__enabled( $enabled )
	{
		$this->enabled	= $enabled;
	}
	/**
	 * [Node] Get Node Title
	 *
	 * @return	string
	 */
	protected function get__title()
	{
		return $this->title;
	}


	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{

		$form->add( new \IPS\Helpers\Form\Text( 'column_title', $this->id ? $this->title : "", TRUE, array( 'maxLength' => 255 ), NULL, NULL, NULL, 'column_title' ) );
		/* Custom column color */
		$form->add( new \IPS\Helpers\Form\YesNo( 'column_usecustombg', $this->id ? $this->bitoptions['usecustombg'] : 0, FALSE, array( 'togglesOn' => array( 'column_custombg', 'column_colsep' ) ), NULL, NULL, NULL, 'column_usecustombg' ) );
		$form->add( new \IPS\Helpers\Form\Color( 'column_custombg', $this->id ? $this->custombg : '#2b2c30', FALSE, array(), NULL, NULL, NULL, 'column_custombg' ) );
		$form->add( new \IPS\Helpers\Form\Radio( 'column_colsep', $this->id ? $this->colsep : "curvedline", FALSE, array( 'options' => array(
			'curvedline'  => 'af_curvedline',
			'borderline' => 'af_borderline',
			'none'	=> 'af_none'
		 ) ), NULL, NULL, NULL, 'column_colsep'));

 			$themes = array();
 			foreach( \IPS\Theme::themes() as $theme )
 			{
 				$themes[ $theme->id ] = $theme->_title;
 			}
			$form->add( new \IPS\Helpers\Form\Select( 'column_themes', $this->id ? ( $this->themes == '*' ? '*' : explode( ',', $this->themes ) ) : '*', NULL, array( 'parse' => 'normal', 'multiple' => TRUE, 'options' => $themes, 'unlimited' => '*', 'unlimitedLang' => 'all' ), NULL, NULL, NULL, 'column_themes' ) );


	}
	/**
	 * [Node] Format form values from add/edit form for save
	 *
	 * @param	array	$values	Values from the form
	 * @return	array
	 */
	public function formatFormValues( $values )
	{

		/* Save bitoptions */
		foreach ( array( 'usecustombg' ) as $k )
		{
			if ( isset( $values[ "column_{$k}" ] ) )
			{
				$this->bitoptions[ $k ] = $values["column_{$k}"];
				unset( $values["column_{$k}"] );
			}
		}

		if ( !$this->id )
		{
			$this->save();
		}

		return $values;
		/* return */
		parent::saveForm( $values );
	}

	/**
	 * [Node] Does the currently logged in user have permission to add a child node to this node?
	 *
	 * @return	bool
	 */
	public function canAdd()
	{
		if ($this->childrenCount(null) >= 4)
		{
			return false;
		}
		return parent::canAdd();
	}

	/**
	 * Search
	 *
	 * @param	string		$column	Column to search
	 * @param	string		$query	Search query
	 * @param	string|null	$order	Column to order by
	 * @param	mixed		$where	Where clause
	 * @return	array
	 */
	public static function search( $column, $query, $order=NULL, $where=array() )
	{
		if ( $column === '_title' )
		{
			$column	= 'column_title';
		}

		if( $order == '_title' )
		{
			$order	= 'column_title';
		}

		return parent::search( $column, $query, $order, $where );
	}

}
